//+------------------------------------------------------------------+
//|                                              Print Functions.mq5 |
//|                                        Copyright  2018, Amr Ali |
//|                             https://www.mql5.com/en/users/amrali |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| Helper macros to print a lot of information from the terminal.   |
//+------------------------------------------------------------------+
#define Param(p1)     p1,
#define Params(p1,p2) p1,p2,

#define PROPERTIES(function, params, enumtype) \
   for(int i=-2; i<USHORT_MAX+1000; ++i) \
     { \
      enumtype prop_id = (enumtype)i; \
      if(StringFind(EnumToString(prop_id),"::")<0) \
         Print(EnumToString(prop_id),"=",function(params##prop_id)); \
     }

//+------------------------------------------------------------------+
//| Prints the account specification to the Experts tab.             |
//+------------------------------------------------------------------+
void PrintAccount()
  {
   if(AccountInfoInteger(ACCOUNT_LOGIN))
     {
      PROPERTIES(AccountInfoInteger,"",ENUM_ACCOUNT_INFO_INTEGER);
      PROPERTIES(AccountInfoDouble,"",ENUM_ACCOUNT_INFO_DOUBLE);
      PROPERTIES(AccountInfoString,"",ENUM_ACCOUNT_INFO_STRING);
     }
  }
//+------------------------------------------------------------------+
//| Prints the terminal properties to the Experts tab.               |
//+------------------------------------------------------------------+
void PrintTerminal()
  {
   if(TerminalInfoString(TERMINAL_COMPANY)!="")
     {
      PROPERTIES(TerminalInfoInteger,"",ENUM_TERMINAL_INFO_INTEGER);
      PROPERTIES(TerminalInfoDouble,"",ENUM_TERMINAL_INFO_DOUBLE);
      PROPERTIES(TerminalInfoString,"",ENUM_TERMINAL_INFO_STRING);
     }
  }
//+------------------------------------------------------------------+
//| Prints the chart properties to the Experts tab.                  |
//+------------------------------------------------------------------+
void PrintChart(const long chart_id)  // 0 means the current chart
  {
   if(ChartSymbol(chart_id)!="")
     {
      PROPERTIES(ChartGetInteger,Param(chart_id),ENUM_CHART_PROPERTY_INTEGER);
      PROPERTIES(ChartGetDouble,Param(chart_id),ENUM_CHART_PROPERTY_DOUBLE);
      PROPERTIES(ChartGetString,Param(chart_id),ENUM_CHART_PROPERTY_STRING);
     }
  }
//+------------------------------------------------------------------+
//| Prints the graphic object properties to the Experts tab.         |
//+------------------------------------------------------------------+
void PrintObject(const long chart_id, const string name)  // 0 means the current chart
  {
   if(ObjectFind(chart_id,name)>=0)
     {
      PROPERTIES(ObjectGetInteger,Params(chart_id,name),ENUM_OBJECT_PROPERTY_INTEGER);
      PROPERTIES(ObjectGetDouble,Params(chart_id,name),ENUM_OBJECT_PROPERTY_DOUBLE);
      PROPERTIES(ObjectGetString,Params(chart_id,name),ENUM_OBJECT_PROPERTY_STRING);
     }
  }
//+------------------------------------------------------------------+
//| Prints the symbol specification to the Experts tab.              |
//+------------------------------------------------------------------+
void PrintSymbol(const string symbol)
  {
   Print(Symbol(),", ",SymbolInfoString(Symbol(),SYMBOL_DESCRIPTION));
   if(SymbolSelect(symbol,true))
     {
      PROPERTIES(SymbolInfoInteger,Param(symbol),ENUM_SYMBOL_INFO_INTEGER);
      PROPERTIES(SymbolInfoDouble,Param(symbol),ENUM_SYMBOL_INFO_DOUBLE);
      PROPERTIES(SymbolInfoString,Param(symbol),ENUM_SYMBOL_INFO_STRING);
     }
  }
//+------------------------------------------------------------------+
//| Prints the position properties to the Experts tab.               |
//+------------------------------------------------------------------+
void PrintPosition(const ulong pos_ticket)
  {
   if((PositionGetInteger(POSITION_TICKET) == pos_ticket && pos_ticket != 0)
      || PositionSelectByTicket(pos_ticket))
     {
      PROPERTIES(PositionGetInteger,"",ENUM_POSITION_PROPERTY_INTEGER);
      PROPERTIES(PositionGetDouble,"",ENUM_POSITION_PROPERTY_DOUBLE);
      PROPERTIES(PositionGetString,"",ENUM_POSITION_PROPERTY_STRING);
     }
  }
//+------------------------------------------------------------------+
//| Prints the order properties to the Experts tab.                  |
//+------------------------------------------------------------------+
void PrintOrder(const ulong order_ticket)
  {
   if((OrderGetInteger(ORDER_TICKET) == order_ticket && order_ticket != 0)
      || OrderSelect(order_ticket))
     {
      PROPERTIES(OrderGetInteger,"",ENUM_ORDER_PROPERTY_INTEGER);
      PROPERTIES(OrderGetDouble,"",ENUM_ORDER_PROPERTY_DOUBLE);
      PROPERTIES(OrderGetString,"",ENUM_ORDER_PROPERTY_STRING);
     }
  }
//+------------------------------------------------------------------+
//| Prints the history deal properties to the Experts tab.           |
//+------------------------------------------------------------------+
void PrintDeal(const ulong deal_ticket)
  {
   if((HistoryDealGetInteger(deal_ticket,DEAL_TICKET) == deal_ticket && deal_ticket != 0)
      || HistoryDealSelect(deal_ticket))
     {
      PROPERTIES(HistoryDealGetInteger,Param(deal_ticket),ENUM_DEAL_PROPERTY_INTEGER);
      PROPERTIES(HistoryDealGetDouble,Param(deal_ticket),ENUM_DEAL_PROPERTY_DOUBLE);
      PROPERTIES(HistoryDealGetString,Param(deal_ticket),ENUM_DEAL_PROPERTY_STRING);
     }
  }
//+------------------------------------------------------------------+
//| Prints the history order properties to the Experts tab.          |
//+------------------------------------------------------------------+
void PrintHistoryOrder(const ulong order_ticket)
  {
   if((HistoryOrderGetInteger(order_ticket,ORDER_TICKET) == order_ticket && order_ticket != 0)
      || HistoryOrderSelect(order_ticket))
     {
      PROPERTIES(HistoryOrderGetInteger,Param(order_ticket),ENUM_ORDER_PROPERTY_INTEGER);
      PROPERTIES(HistoryOrderGetDouble,Param(order_ticket),ENUM_ORDER_PROPERTY_DOUBLE);
      PROPERTIES(HistoryOrderGetString,Param(order_ticket),ENUM_ORDER_PROPERTY_STRING);
     }
  }

//+------------------------------------------------------------------+
//| Print enumeration.                                               |
//+------------------------------------------------------------------+
//| Examples:                                                        |
//|                                                                  |
//|   PrintEnum<ENUM_TIMEFRAMES>();                                  |
//|   PrintEnum<ENUM_ACCOUNT_INFO_DOUBLE>();                         |
//|   PrintEnum<ENUM_FP_CLASS>();      // lowest enum value          |
//|   PrintEnum<ENUM_CHART_EVENT>();   // highest enum value         |
//+------------------------------------------------------------------+
template <typename T>
void PrintEnum(T dummy = -1)
  {
   Print(typename(T));
   Print("  {");
   for(int i = -2; i < USHORT_MAX + 1000; i++)
      if(StringFind(EnumToString((T)i),"::")<0)
         printf("   %s = %d,",EnumToString((T)i),i);
   Print("  };");
  }

//+------------------------------------------------------------------+
//| Print sructure variable.                                         |
//+------------------------------------------------------------------+
template <typename T>
void PrintStruct(T& struct_var)
  {
   T arr[1]; // one-element array
   arr[0] = struct_var;
   ArrayPrint(arr);
  }


#undef Param
#undef Params
#undef PROPERTIES


//+------------------------------------------------------------------+
//| Test script                                                      |
//+------------------------------------------------------------------+
void OnStart()
  {
   PrintAccount();
   PrintTerminal();
   PrintChart(0);
   PrintObject(0,"MyBitmap");
   PrintSymbol(_Symbol);
   PrintPosition(925571424);
   PrintDeal(609261076);
   PrintHistoryOrder(925571424);

   PrintEnum<ENUM_TIMEFRAMES>();

   MqlDateTime dt;
   TimeToStruct(TimeCurrent(), dt);
   PrintStruct(dt);
  }


//+------------------------------------------------------------------+
//| Test script 2                                                    |
//+------------------------------------------------------------------+
//void OnStart()
//  {
//   if(HistorySelect(0, TimeCurrent()))
//     {
//      int max = fmin(HistoryDealsTotal(), 3);  // limit to 3 deals
//      for(int i = 0; i < max; ++i)
//        {
//         // print the history deal properties to the Experts tab
//         PrintDeal(HistoryDealGetTicket(i));
//         Print("--------");
//        }
//     }
//  }
